function obj = ctransposeDisk(obj)
% ctransposeDisk - transpose deflated matrix (on disk method)
%
% This method transposes  a deflated matrix. It uses the on-disk method,
% which means that data are dumped to temporary files and a new matrix is
% built from these files. This is generally faster then the in-memeory
% method (see ctransposeMemory).
%
% CALL:
%  T = transpose(obj)
% 
% INPUT:
%  obj:     <object>  deflate object
% 
% OUTPUT:
%  obj:       <object> transposed deflate object
%
% TEST
%     adata=rand(200000,20);
%     nw=now;
%     a=deflate(adata);
%     dprintf('T1=%f',86400*(now-nw));%  bdata=full(ctransposeDisk(a));
%     bdata=full(a');
%     dprintf('T2=%f',86400*(now-nw));
%     is_eq(adata',bdata);
%     ==> T2=19 sec
% 
% APPROACH
%     Suppose M is to large to fit in memory (which is generally the reason
%     for storing M as a deflated matrix. In this case the matlab transpose
%     method can not be used. Instead we split up M is sub matrices that are
%     small enough to transpose separately, for example:
%         M=[A,B,C,D]
%     The submatrices are first dumped to disk, then the original matrix is
%     deleted and a new matrix is built from these submatrices:
%         M'=[A';B';C';D']
%
%     To economize in the deflate phase. The matrices A-D are split.
%             
% 

% 
% EXPERIMENT
% if 0
% a=int16(1000*rand(10000,2000));
% aa=deflate(a);
% tic
% aa2=ctransposeDisk(aa);
% dprintf('ctransposeDisk: %f',toc)
% a=a';
% is_eq(a,full(aa2));
% end

%     a=rand(10^7,1);
%     tic;aa=deflatev(a);TIME_CMPR=toc
%     tic;aaa=aa(:);TIME_UCMPR=toc
%     F=TIME_CMPR/TIME_UCMPR %typical: 2.8780
%     is_eq(a,aaa)
%     
%     a=int16(100*rand(2000,100)); %or any ather type
%     tic;aa = compress(a);TIME_CMPR=toc
%     tic;aaa =uncompress(aa,size(a),class(a));TIME_UCMPR=toc
%     F=TIME_CMPR/TIME_UCMPR %typical: 12.68)

if length(obj.size)>2
    error('Transpose on ND array is not defined.');
end
% length(obj.size)==2;
if any(obj.size==1)||any(obj.size==0)
    %rearranging elements not needed
    obj.size=[obj.size(2),obj.size(1)];
    return
end

%rearranging elements is needed
S=defineSubmatrix(obj);

try
    % FOR ALL J:
    % FOR ALL I:
    hwait=waitbar(0,'Transpose: read matrix content');
    F=writeTimeFactor(obj.data);
    N=(1+F)*numel(S);
    n=1; %waitbar data
    %save data in order S11,S21,S31.. S12,S22,... etc
    for J=1:size(S,2)
        %read the data for this column
        M=subsref(obj,substruct('()',{':',S(1,J).rngcol}));
        %write the data to disk
        for I=1:size(S,1)
            D=M(S(I,J).rngrow,:);
            D=D';
            save(S(I,J).fname,'D','-v6');
            S(I,J).ondisk=true;
            waitbar(n/N,hwait);
            n=n+1;
        end
    end

    %everything has now been read and saved to disk
    %free the memory that is stored in current deflated vector

    %create new deflated vector that holds data
    newvector=reset(obj.data); 
	%change segment size to matrix height for efficiency
	newvector=set(newvector,'segm',max(obj.size(2),16384));  
    newvector=set(newvector,'N',prod(obj.size));

    %define new size
    obj.size=[obj.size(2),obj.size(1)];

    %store data in new compressed matrix
    % FOR ALL J:
    % FOR ALL I:
    %compress data in order S11,S12,S13.. S21,S22,... etc
    waitbar(n/N,hwait,'Transpose: store matrix content');
    ELEMENT=feval(get(obj.data,'class'),0);
    for I=1:size(S,1)%initialize matrix of right size and type
        M=repmat(ELEMENT,obj.size(1),length(S(I,1).rngrow));
        for J=1:size(S,2) %WIJZ ZIJPP MAR 2006
            load(S(I,J).fname);%Load D
            delete(S(I,J).fname);
            S(I,J).ondisk=false;
            M(S(I,J).rngcol,:)=D;
            waitbar(n/N,hwait);
            n=n+F;
        end
        obj=subsasgn(obj,substruct('()',{':',S(I,1).rngrow}),M);
    end
    close(hwait);
catch
    cleanupDisk(S); %delete the temporary files
    if ishandle(hwait)
        close(hwait);
    end
    rethrow(lasterror);
end
%__________________________________________________________________________
function S=defineSubmatrix(obj)
%Define reading strategy

%Number od elments depends on available memory (in bytes):
b=availableMemory;

%Maximum number of elements that can be stored (take 4 bytes per index into
%account)
numElm=b/(8+8+4+4+2+sizeof(obj)); 
numElm=b/(4+4+2+sizeof(obj)); 
numElm=numElm;
%Memory bottlenecks in:
%    sub2indVector/repmat (4+4+4)
%    groupIndex/unique (8+8+4+4+2+sizeof(obj)) %SOLVED!

%How many columns can we read?
maxRead=floor(numElm/obj.size(1)); %do not read more then this number of columns at a time
maxWrite=floor(numElm/obj.size(2)); %do not read more then this number of rows at a time

if maxRead<1 || maxWrite<1
    error('Unsufficient memory to transpose matrix');
end
%Required action: read data, maxRead columns ata a time. 
%Transpose them. Save them but width should not exceed maxWrite.

%Define the submatrices 
COL=1;
% Read columns
for cStart=1:maxRead:obj.size(2)
    cRange=cStart:min(cStart+maxRead-1,obj.size(2));    
    ROW=1;
    for rStart=1:maxWrite:obj.size(1)
        rRange=rStart:min(rStart+maxWrite-1,obj.size(1));
        S(ROW,COL)=struct('rngrow',rRange,...
            'rngcol',cRange,...
            'fname',extensie(tempname,'mat'),...
            'ondisk',false);
        ROW=ROW+1;
    end
    COL=COL+1;
end
disp(size(S));
%__________________________________________________________________________
function cleanupDisk(S)
%delete any remaining the temporary files

if ~isfield(S,'fname');
    return
end
for k=1:numel(S)
    this=S(k);
    if this.ondisk
        if exist(this.fname,'file')
            delete(this.fname)
        end
    end
end

