function obj = ctransposeMemory(obj)
% ctransposeMemory - transpose deflated matrix (in-memory method)
%
% This method transposes  a deflated matrix. It uses the in-memory method,
% which means that during the process of rearranging the matrix elements
% data are stored in-memory and no temporary files are needed.  The
% downside is that this method is not as fast as the on-disk method (see
% ctransposeDisk).
%
% CALL:
%  T = transpose(obj)
% 
% INPUT:
%  obj:     <object>  deflate object
% 
% OUTPUT:
%  obj:       <object> transposed deflate object
%
% transpose - transponeer de inhoud van een deflate object
%
% CALL:
%  T = transpose(obj)
%
% INPUT:
%  obj:     <object> een deflate object
%
% OUTPUT:
%  obj:       <object> het getransponeerde deflate object
%
% TEST
%     adata=rand(200000,20);
%     nw=now;
%     a=deflate(adata);
%     dprintf('T1=%f',86400*(now-nw));
%     bdata=full(a');
%     dprintf('T2=%f',86400*(now-nw));
%     is_eq(adata',bdata);


if length(obj.size)>2
    error('Transpose on ND array is not defined.');
end
% length(obj.size)==2;
if any(obj.size==1)||any(obj.size==0)
    %rearranging elements not needed
    obj.size=[obj.size(2),obj.size(1)];
    return
end
%rearranging elements is needed
% obj.data=transposeElementsOnDisk(obj.data,obj.size);
obj.data=transposeElements(obj.data,obj.size);
obj.size=[obj.size(2),obj.size(1)];
%__________________________________________________________________________
function newvector=transposeElements(oldvector,oldSize)
% INPUT
%     oldvector: deflatev object holding data for matrix of size oldSize
%     oldSize: non transposed size of matrix

%Because decompressing is faster then compressing, minimize the time needed
%to compress

%Compute the maximum number of elements to write
CLASS      = get(oldvector,'class');
%MEMORY REQUIRED:
%  rng (UINT32)
%+ oldrng (UINT32)
%+ temporary buffer in deflatev object
%  = unsorted index (UINT32)
%    offsets (UINT32)
%    block index (UINT32)
%    data (sizeof(CLASS))
%+ temporary copies in sort algorithm (16 bytes)
% Total : (5 *4 bytes + sizeof(CLASS)+16) * NUMELMS
MAXELM     = (64*10^6)/(sizeof(CLASS)+36);
SEGSIZE    = get(oldvector,'segmentSize');
MAXSEGMENT =floor(MAXELM/SEGSIZE); %maximimum number of segment to write in one go

%initialize new vector
newvector=reset(oldvector);
newvector=set(newvector,'N',prod(oldSize));

if 0
    %assign elements of new vector in loop
    strt=1;
    N=prod(oldSize);
    while strt<=N

        last=min(strt+MAXSEGMENT*SEGSIZE-1,N);
        dprintf('copy range %10d - %10d from %d',strt,last,N);
        rng=uint32(strt):uint32(last); %elements required for new matrix
        oldrng=transposeIndex(rng,oldSize(1),oldSize(2));
        newvector(rng)=oldvector(oldrng);
        strt=last+1;
    end
else
    newSize=fliplr(oldSize);
    %define indices to NEW matrix
    [rowBgn,rowEnd,colBgn,colEnd]= strategyTranspose(newSize);
    hwait=waitbar(0,'Create transpose matrix');
    for B=1:numel(rowBgn)
        waitbar(B/numel(rowBgn),hwait);
        %get linear in NEW matrix index that corresponds to subscripts
        rng=sub2indVector(newSize,rowBgn(B):rowEnd(B),colBgn(B):colEnd(B));
        %derive corresponding indices in OLD matrix
        oldrng=transposeIndex(rng,oldSize(1),oldSize(2));
        %assign OLD elements to NEW matrix
        newvector(rng)=oldvector(oldrng);
    end
    close(hwait)
end


%__________________________________________________________________________
function oldIndex = transposeIndex(trIndex,oldH,oldW)

STEP=5*10^5;
L=uint32(length(trIndex));
first=uint32(1);
oldIndex=zeros(size(trIndex),'uint32');

%do work in small portions so that it fits in memory
while first<=length(trIndex)
    last=min(first+STEP,L);
    rng=first:last;
    oldIndex(rng) = transposeIndex_compute(trIndex(rng),oldH,oldW);
    first=last+1;
end

%__________________________________________________________________________
function oldIndex = transposeIndex_compute(trIndex,oldH,oldW)
% transposeIndex - bepaal de index in een getransponeerde matrix
%
% INPUT
%     trIndex: required index in transpose matrix (vector)
%     oldH: height of matrix before transpose
%     oldW: height of matrix before transpose
% OUTPUT
%     oldIndex: corresponding index in old matrix (vector)
%
% APPROACH
% The elements in a matrix with Width and Height are ordere as follows
%     1  H+1 .. (W-1)H+1
%     2  H+2 .. (W-1)H+2
%     3  H+3 .. (W-1)H+3
%     .. ..  .. ..
%     H  2H  .. WH
% The linear index is hence 1,2,3 .. WH
% After transpose the linear index is:
%     1  H+1 .. (W-1)H+1    2  H+2 .. (W-1)H+2    3  H+3 .. (W-1)H+3 ..   H  2H  .. WH

% Hence: oldindex can be read from the following table:
%     trIndex : 1  2      W           ..
%     oldIndex: 1  H+1 .. (W-1)H+1    2  H+2 .. (W-1)H+2    3  H+3 .. (W-1)H+3 ..   H  2H  .. WH
%     ==        1  1       1          2  2       2          3  3       3       ..   W  W      W
%      +        0  H      (W-1)H      0  H      (W-1)H      0  H      (W-1)H        0  H     (W-1)H
% To access element i of the transpose matrix (by reading in old matrix)
% apply the following arithmetic
trIndex=double(trIndex); %WARNING arithmetic MUST be done in double!!
oldIndex= 1+floor((trIndex-1)/oldW) + rem(trIndex-1,oldW)*oldH;
oldIndex= uint32(oldIndex);
% index = (index - 1)*m + floor((index - 1)/n);
% index = mod(index,m*n)+1;
%_________________________________________________________
function [rowBgn,rowEnd,colBgn,colEnd]= strategyTranspose(newSize)
%generate transpose strategy
% 
% INPUT
%     newSize: new size of matrix
% OUTPUT
%     rowBgn
%     rowEnd
%     colBgn
%     colEnd


%read more columns because reading is faster
%read at least 600 columns
stepCol=min(600,newSize(2));
Nmax=600*1200;
%make stepRow as large as possible within bounds
stepRow=   ceil(Nmax/stepCol);
if stepRow>newSize(1)
    stepRow = newSize(1);
    stepCol=   ceil(Nmax/stepRow);
end

%stepRow: length of row range
%stepCol: length of col range
rowBgnVec=1:stepRow:newSize(1);
rowEndVec=min(rowBgnVec+stepRow-1,newSize(1));
colBgnVec=1:stepCol:newSize(2);
colEndVec=min(colBgnVec+stepCol-1,newSize(2));
%rowBgnVec(I),rowEndVec(I): vector of length I
%colBgnVec(J),colEndVec(J): vector of length J

%now expand to I x J matrices:

%expand to matrices
rowBgn=uint32(rowBgnVec(:)*ones(1,length(colBgnVec)));
rowEnd=uint32(rowEndVec(:)*ones(1,length(colBgnVec)));
colBgn=uint32(ones(length(rowBgnVec),1)*colBgnVec);
colEnd=uint32(ones(length(rowBgnVec),1)*colEndVec);