function obj=deflatev(varargin)
% deflatev - constructor for deflated vector
% 
% A deflated vector is used for storage of large non-sparse vectors of up
% to 2^32 elements.
% 
% CALL
%     v=deflatev(V), OR:
%     v=deflatev(V,<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>,..), OR:
%     v=deflatev(<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>,..)
%
% INPUT
%     V(N)      : numeric vector (matrices are converted to vectors),
%                 deflated vector, or structure with equivalent fields
% 
%     <PROPERTY>,<VALUE>: property value pair. Valid combinations are
%     'emptyvalue': value assumed for elements that have not yet been
%                initialized. Defaults to zero or blank (depending on
%                storage class)
%                Defaults to double(0). 
%     'N'      : Number of elements in deflated vector.
%                Defaults to number of elements of V
%     'mode'   : Storage mode: memory, compressed, disk
%                Defaults to compressed
%                compressed: equal to memory, but now the data blocks are
%                        compressed in memory. This expands the maximum 
%                        storage size considerably. If random access is
%                        required the CPU costs may be high.
%                memory: data is stored in memory in contiguous blocks of
%                        size "segmentSize". This expands the maximum
%                        storage size typically with a factor 3. compared
%                        to storage in contiguous blocks. The CPU costs are
%                        moderate.
%                disk:   TO BE IMPLEMENTED the data blocks are stored on disk. This is faster
%                        then compression, but also more vulnarable. Copies
%                        of the object point to the same file.
%     'segmentSize': smallest blocksize to break up vector
%                    Defaults to 100000

% 
% OUTPUT
%     v: compressed vector that contains the elements of V
%        v is promarily intended as a storage object
%
% INTERNAL DATASTRUCTURE
%     |
%     +----segmentSize  double 
%     +----storemode    double {1,2}
%     +----dataclass    CHAR
%     +----emptyvalue   type depends on datatclass
%     +----segment(STRUCT 1xP)
%          +----dat    UINT8 n x1
%          +----range  UINT32 1x2
%     
% EXAMPLE
%     a=deflatev([])
%     N=10^6;
%     for k=1:N:10^8
%         disp(k)
%         rng=(k:k+N-1);
%         a(rng)=rng;
%     end
%
% SEE ALSO
%     testdeflatev
% 
% Note on redundancy between V and emptyvalue
% 1. V specified: 
%    1.1 emptyvalue specified:
%        When vector is extended, it will be padded
%        with "emptyvalue". class(emptyvalue) must equal class(V)
%    1.2 emptyvalue NOT specified: 
%        emptyvalue will default to emptyDefault(class(V)).
% 2. V NOT specified:
%    2.1 emptyvalue specified: 
%        All unitialized elemends (implied by size property) equal
%        emptyvalue.
%    2.2 emptyvalue NOT specified: 
%        emptyvalue will default to double(0)
%   

% Note: property "class" is no longer allowed

% Benchmark
% var=1:10^6;
% tic;a=deflatev(var);dprintf('Deflate: %f',toc);
% tic;save('test','var');dprintf('Save: %f',toc);delete('test.mat');
% tic;save('test','-v6','var');dprintf('Save -v6: %f',toc);delete('test.mat');
    
%-----------------------
%No arguments
%-----------------------
if nargin==0
    obj=deflatev([]);
    return
end

%-----------------------
%1 or more arguments
%-----------------------
% Special case
% -1- implement deflate(obj), with obj of class deflate
if isa(varargin{1},'deflatev')
    obj=varargin{1};
    if nargin>1
        error('When first argument is deflatev object extra arguments are not allowed');
    end
    return
end


%-----------------------------------------------------------
%1 or more arguments first argument is not deflatev object
%-----------------------------------------------------------
if rem(nargin,2)==1
    %The first argument is a datavector

    %derive 'class' and 'N' from data
    V=varargin{1};
    %if input data is specified, empty value must be of the same class
    inputClass=class(V); 
    argused=1;
    ValidProps={'mode','segmentSize','emptyvalue'};
else
    %create a one element deflate vector object
    V         =[];
    inputClass='';
    argused   =0;
    ValidProps={'mode','segmentSize','emptyvalue','size','N'}; %size argument is ignored
end

if isstruct(V)
    %this might be a deflatev object converted to structure
    %check for required fields
    
    %     supply storemode option if missing
    if ~isfield(V,'storemode')
        V.storemode=strmatch('compressed' ,storeModes); %1,2,of 3;
    end
    
    aa=struct(deflatev(1));
    rqfields=fieldnames(aa);
    for k=1:length(rqfields)
        fld=rqfields{k};
        if isfield(V,fld)
            S.(fld)=V.(fld);
        else
            S.(fld)=aa.(fld);
            dprintf('Field "%s" missing: use default value',fld);
        end
    end
else
    %datavector V is specified or initialized at []
    opt.segmentSize  = 100000;
    opt.N            = numel(V);
    opt.mode         = 'compressed';
    if isempty(inputClass)
        %V not defined, default is double        
        opt.emptyvalue   = emptyDefault('double');
    else
        %provide default value for emptyvalue
        opt.emptyvalue   = emptyDefault(inputClass);
    end
  
    %check specified properties (ignore option "emptyvalue")
    opt=varargin2struct(opt,ValidProps,varargin{argused+1:end});

    if ~isempty(inputClass)       
        %check consistency    
        if ~strcmp(inputClass,class(opt.emptyvalue))
            error('Class of property "emptydata" (%s) must match class of specified data (%s)',...
                class(opt.emptyvalue),inputClass);
        end
    end
    %check specified multiple choice values
    opt.mode =getproperty(opt.mode,storeModes);

    %set segment size
    S.segmentSize  = opt.segmentSize;
    S.storemode    = strmatch(opt.mode ,storeModes); %1,2,of 3

    %store data in multiple segments
    %initialize first (empty) segment
    S.segment = emptysegment(S.storemode);

    %copy class from data
    S.emptyvalue   = opt.emptyvalue;
    S.dataclass    = class(S.emptyvalue); %redundant for ease of notation
end
%initialize object
obj=class(S,'deflatev');

%---------------------------------
%copy data to object
%---------------------------------

if isempty(V)
    %Deal with syntax: deflatev('N',N)
    if opt.N==0
        return
    end
    %initialze vector with zero elements
    Z=emptyData(obj,1);
    
    %set 1 element to zero to force right size
    obj=subsasgn(obj, substruct('()',{opt.N}),Z); 
    
    %Only the last block is initialized. Earlier block contain
    %     data=[] and range=[]. By convention this means that they are
    %     all empty
    return
end
if isstruct(V)
    %all information is already stored in S
    return
end
%copy elements of V to object

%set 1 element of vector to force right size
obj=subsasgn(obj, substruct('()',{opt.N}),V(end));

%copy elements to datavector
obj=subsasgn(obj, substruct('()',{':'}),V(:));
%__________________________________________________________________________
function val=emptyDefault(str)

switch str
    case 'char'
        val=' ';
    case 'logical'
        val=false;
    case {'double','single','int8','uint8','int16','uint16','int32','uint32','int64','uint64'};
        val=zeros(1,1,str);
    otherwise
        error('variables of type %s can not be deflated',str);
end
    

%PUBLIC
% deflatev.m    
% full.m             OK 
% length.m           OK
% set.m              OK
% subsasgn.m         OK
% writeTimeFactor.m  OK
% display.m          OK
% get.m              OK
% numel.m            OK
% size.m             OK
% subsref.m          OK
% end.m              OK
% getrange.m         
% putrange.m         
% sizeof.m           OK
% testdeflatev.m     
% 
% 
% PRIVATE
% acceptedClasses.m OK
% getOffsets.m      OK
% range2blk.m       OK
% storeModes.m      OK
% emptyData.m       OK
% getSegment.m      OK
% segmentData.m     OK 
% storeSegment.m    OK 
% emptysegment.m    OK 
% makeComplete.m    DELETE 
% sortedindex.m     OK 
% verifyRange.m     OK 




