function obj=deflatevector(varargin)
% deflatev - constructor for deflated vector
% 
% A deflated vector is used for storage of large non-sparse vectors of up
% to 2^32 elements.
% 
% CALL
%     v=deflatev(V), OR:
%     v=deflatev(V,<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>,..), OR:
%     v=deflatev(<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>,..)
%
% INPUT
%     V(N)      : numeric vector (matrices are converted to vectors),
%                 deflated vector, or structure with equivalent fields
% 
%     <PROPERTY>,<VALUE>: property value pair. Valid combinations are
%     'class':   class of stored elements: char, or any numeric Matlab type
%                Defaults to double. 
%                Use class(V) if V is specified.
%     'N'      : Number of elements in deflated vector.
%                Defaults to number of elements of V
%     'mode'   : Storage mode: memory, compressed, disk
%                Defaults to compressed
%                compressed: equal to memory, but now the data blocks are
%                        compressed in memory. This expands the maximum 
%                        storage size considerably. If random access is
%                        required the CPU costs may be high.
%                memory: data is stored in memory in contiguous blocks of
%                        size "segmentSize". This expands the maximum
%                        storage size typically with a factor 3. compared
%                        to storage in contiguous blocks. The CPU costs are
%                        moderate.
%                disk:   TO BE IMPLEMENTED the data blocks are stored on disk. This is faster
%                        then compression, but also more vulnarable. Copies
%                        of the object point to the same file.
%     'segmentSize': smallest blocksize to break up vector
%                    Defaults to 100000
%     'emptyvalue': value assumed for elements that have not yet been
%                   initialized. Defaults to zero or blank (depending on
%                   datatype)
% 
% OUTPUT
%     v: compressed vector that contains the elements of V
%        v is promarily intended as a storage object
%
% INTERNAL DATASTRUCTURE
%     segmentSize
%     storemode
%     dataclass
%     segment
%     +----dat
%     +----range
%     
% EXAMPLE
%     a=deflatev([])
%     N=10^6;
%     for k=1:N:10^8
%         disp(k)
%         rng=(k:k+N-1);
%         a(rng)=rng;
%     end
%
% SEE ALSO
%     testdeflatev
%-----------------------
%No arguments
%-----------------------
if nargin==0
    obj=deflatevector([]);
    return
end

%-----------------------
%1 or more arguments
%-----------------------
if isa(varargin{1},'deflatevector')
    obj=varargin{1};
    if nargin>1
        error('When first argument is deflatevector object extra arguments are not allowed');
    end
    return
end

%-----------------------------------------------------------
%1 or more arguments first argument is not deflatev object
%-----------------------------------------------------------
if rem(nargin,2)==1
    %The first argument is a datvector
    %derive 'class' and 'N' from data
    V=varargin{1};
    argused=1;
    ValidProps={'mode','segmentSize','emptyvalue'};
else
    %create a one element deflate vector object
    V         =[];
    argused   =0;
    ValidProps={'class','N','mode','segmentSize','emptyvalue'};
end

if isstruct(V)
    %this might be a deflatev object converted to structure
    %check for required fields
    
    %     supply storemode option if missing
    if ~isfield(V,'storemode')
        V.storemode=strmatch('compressed' ,storeModes); %1,2,of 3;
    end
    
    aa=struct(deflatevector(1));
    rqfields=fieldnames(aa);
    for k=1:length(rqfields)
        fld=rqfields{k};
        if ~isfield(V,fld)
            error('Field "%s" missing',fld);
        end
        S.(fld)=V.(fld);
    end
else
    opt.segmentSize  = 100000;
    opt.class        = class(V);
    opt.N            = numel(V);
    opt.mode         = 'compressed';
    %check specified properties (ignore option "emptyvalue")
    opt=varargin2struct(opt,ValidProps,varargin{argused+1:end});

    %check specified multiple choice values
    opt.mode=getproperty(opt.mode,storeModes);
    opt.class=getproperty(opt.class,acceptedClasses);

    %set segment size
    S.segmentSize  = opt.segmentSize;
    S.storemode    = strmatch(opt.mode ,storeModes); %1,2,of 3

    %store data in multiple segments
    %initialize first (empty) segment
    S.segment = emptysegment(S.storemode);

    %copy class from data
    S.dataclass        = opt.class;
    
    %class of object is now known. Create default empty value
    switch S.dataclass
        case 'char'
            opt.emptyvalue=' ';
        case 'logical'
            opt.emptyvalue=false;
        otherwise
            opt.emptyvalue=zeros(1,1,S.dataclass);
    end
    %read  options again including option "emptyvalue"
    opt=varargin2struct(opt,ValidProps,varargin{argused+1:end});
    S.emptyvalue=opt.emptyvalue;
    if ~strcmp(class(S.emptyvalue),S.dataclass)
        error('Assertion failed: emptyvalue and storage class not consistent');
    end
end
%initialize object
obj=class(S,'deflatevector');

%---------------------------------
%copy data to object
%---------------------------------

if isempty(V)
    %Deal with syntax: deflatev('N',N)
    if opt.N==0
        return
    end
    %initialze vector with zero elements
    Z=emptyData(obj,1);
    
    %set 1 element to zero to force right size
    obj=subsasgn(obj, substruct('()',{opt.N}),Z); 
    
    %Only the last block is initialized. Earlier block contain
    %     data=[] and range=[]. By convention this means that they are
    %     all empty
    return
end
if isstruct(V)
    %all information is already stored in S
    return
end
%copy elements of V to object

%set 1 element of vector to force right size
obj=subsasgn(obj, substruct('()',{opt.N}),V(end));

%copy elements to datavector
obj=subsasgn(obj, substruct('()',{':'}),V(:));


%PUBLIC
% deflatevector.m    
% full.m             OK 
% length.m           OK
% set.m              OK
% subsasgn.m         OK
% writeTimeFactor.m  OK
% display.m          OK
% get.m              OK
% numel.m            OK
% size.m             OK
% subsref.m          OK
% end.m              OK
% getrange.m         
% putrange.m         
% sizeof.m           OK
% testdeflatev.m     
% 
% 
% PRIVATE
% acceptedClasses.m OK
% getOffsets.m      OK
% range2blk.m       OK
% storeModes.m      OK
% emptyData.m       OK
% getSegment.m      OK
% segmentData.m     OK 
% storeSegment.m    OK 
% emptysegment.m    OK 
% makeComplete.m    DELETE 
% sortedindex.m     OK 
% verifyRange.m     OK 




