function [data]=getclosedrange(obj,rng)
% getclosedrange - get all elements in closed range or ranges
% 
%CALL
%  [data]=getclosedrange(obj,rng)
%
%INPUT
%  obj: compressed vector
%  rng(2,N): set of ranges, specified in pairs
%
%OUTPUT
%  data: data that corresponds to rng
%
%LARGEST MEMORY CLAIM
%    size of datavector that is retrieved


%Check size of range: not empty and even number of elements
rng=verifyRange(rng); 
%- empty ranges deleted 
%- adjacent ranges have been merged
%- reshaped to 2 xN matrix

if isempty(rng)
    data=feval(get(obj,'class'),[]);
    return
end
    
if any(rng(2,:)>numel(obj))
    error('Index exceeds matrix dimensions');
end

bSize=obj.segmentSize;
[M,nElm]=range2blk(rng,bSize);
%M:
%     row 1: start index (relative to compressed vector)
%     row 2: end index   (relative to compressed vector)
% 	  row 3: segment index (sorted)
%     row 4: start index (in returned vector)
%     row 5: end index   (in returned vector)

unblks=unique(M(3,:)); %unique segment indices

%initialize output
data=emptyData(obj,nElm);
	
for blk=unblks
	%visit each segment only once
	% segment exists
	segment=obj.segment(blk);
	if ~isempty(segment.dat) %else: no action needed: leave zeros
		f_rng=find(blk==M(3,:)); %indices of ranges that are contained in this segment
		startreqMulti = min(M(1,f_rng)); %first requested item (multiple ranges)
		endreqMulti   = max(M(2,f_rng)); %last requested item (multiple ranges)
		startSeg      = segment.range(1)+(blk-1)*bSize; %first available item (this segment)
		endSeg        = segment.range(2)+(blk-1)*bSize;  %last available item (this segment)
		
		start_overlap = max(startreqMulti,startSeg); %first common element (multiple ranges)
		end_overlap   = min(endreqMulti,endSeg);     %last common element (multiple ranges)
		%NOTE!!: All indices refer to the stored/deflated vector
		
		if start_overlap<=end_overlap 
			%else: no overlap==> do nothing and return values set by emptyData
			
			%there is overlap with requested data for at least 1 range ==> inflate data
			%retrieve data only once 
			
			segdata=segmentData(obj.storemode,segment,obj.dataclass);
			%NOTE: these data refer to startSeg-endSeg 
			
			%assign the data to the ranges "f_rng", 1 at a time
			for ff=f_rng
				%get data for 1 range within one block
				start_overlap_ff = max(M(1,ff),start_overlap);  %first common element (this range)
				end_overlap_ff   = min(M(2,ff),end_overlap);    %last common element (this range)
				
				if start_overlap_ff<=end_overlap_ff 
					%else no overlap ==> do nothing and return values set by emptyData
					
					%Compute offsetts in output vector. Note: M(1,ff)
					%corresponds to M(4,ff)
					%IF start_overlap_ff==M(1,ff) ==> use M(4,ff)
					%ELSE                         ==> skip start_overlap_ff-M(1,ff) 
					start_out = M(4,ff)+start_overlap_ff-M(1,ff);
					end_out   = M(5,ff)+end_overlap_ff  -M(2,ff);
					%Compute offsets in segment
					%IF start_overlap_ff==startSeg ==> start at 1
					%ELSE                          ==> skip
					%      start_overlap_ff-startSeg
					data(start_out:end_out)=...
						segdata((1+start_overlap_ff-startSeg):(1+end_overlap_ff-startSeg));
				end
			end
		end
	end		
end

function dummy

nElmCol=(rng(2,:)-rng(1,:))+1; %number of elements per colum of rng
extStart=[1,1+cumsum(nElmCol(1:end-1))]; %start index in external output vector per column
extEnd  =cumsum(nElmCol); %start index in external output vector per column
nElm=sum(nElmCol); %total number of elements in target vector

NOW=now; %plot waitbar if time exceeds 5 sec.

if size(rng,2)>1
	%initialize output
	data=emptyData(obj,nElm);
	
	Bsize=obj.segmentSize;
	unblks=range2blk(rng,Bsize);

	% -2- expand the blocks one at a time
	hwait=[];
	TGAP=5/(1440*60);
	for blk=unblks'
		hwait=waitbarc((now-NOW)>TGAP,k/size(rng,2),hwait,'Inflating');
		% segment exists
		segment=obj.segment(blk);

		if ~isempty(segment.dat) %else: no action needed: leave zeros
			
			
			%retrieve data
			segdata=segmentData(obj.storemode,segment,obj.dataclass);
			%range in full vector represented by these data
			segrange=[1+(blk-1)*Bsize,(blk-1)*Bsize+length(segdata)];

			% -3- assign elements of expanded blocks to output vector
			for k=1:size(rng,2)
				%find overlapping elements
				firstcommon = max(segrange(1),rng(1,k));
				lastcommon  = min(segrange(2),rng(2,k));
				%firstcommon and lastcommon are indices in full vector
				if firstcommon<=lastcommon
					%get index in segdata
					segIndx=(firstcommon-segrange(1)+1):(lastcommon-segrange(1)+1) ;
					%get external index
					extIndx=((firstcommon-rng(1,k)):(lastcommon-rng(1,k)))+extStart(k);
					%assign data
					data(extIndx)=segdata(segIndx);
				end
			end
			%close waitbar if it was created
		end
		close(hwait);
	end
	
	for k=1:size(rng,2)
		data(extStart(k):extEnd(k))=get1range(obj,rng(1,k),rng(2,k));
	end
else
	data=get1range(obj,rng(1),rng(2));
end

function data=get1range(obj,first,last)
%retrieve data for 1 closed range

%prepare for loop
NOW=now;
TGAP=3/(1440*60);
hwait=[];
firstSegment=1+floor((first-1)/obj.segmentSize);
lastSegment =1+floor((last-1)/obj.segmentSize);

%initialize output
data=emptyData(obj,1+last-first);

%now retrieve data from all segments and export data
for blk=firstSegment:lastSegment
    hwait=waitbarc((now-NOW)>TGAP,(1+blk-firstSegment)/(1+lastSegment-firstSegment),hwait,'Inflate');
    
    if ~isempty(obj.segment(blk).dat) %else: no action needed: leave zeros
        %retrieve data
		%get range stored in this segment
        rng        = (blk-1)*obj.segmentSize +(obj.segment(blk).range(1):obj.segment(blk).range(2));
		%Get overlap with requested range
		segstart=(blk-1)*obj.segmentSize;
		blkStrt=max(first,segstart+obj.segment(blk).range(1));
		blkEnd =min(last,segstart+obj.segment(blk).range(2));
		
		if blkEnd>=blkStrt
			% nonempty data to assign
			
			%get index of retrieved data in segment
			from=1+blkStrt-obj.segment(blk).range(1)-segstart;
			to  =1+blkEnd-obj.segment(blk).range(1)-segstart;

			%likewise reletive to vector "data"
			d_from=1+blkStrt-first;
			d_to  =1+blkEnd-first;

			rawdata = segmentData(obj.storemode,obj.segment(blk),obj.dataclass);
			data(d_from:d_to)=rawdata(from:to);
		end
    end
end
close(hwait);

% 
% %get the data out in an economical manner: do not expand blocks more then
% %once
% 
% % -1- make a list of blocks that should be expanded
% % -2- expand the blocks one at a time
% % -3- assign elements of expanded blocks to output vector
% 
% % -1- make a list of blocks that should be expanded
% Bsize=obj.segmentSize;
% unblks=range2blk(rng,Bsize);
% 
% % -2- expand the blocks one at a time
% hwait=[];
% TGAP=5/(1440*60);
% for blk=unblks'    
%     hwait=waitbarc((now-NOW)>TGAP,k/size(rng,2),hwait,'Inflating');
%     % check if segment exists
%     if length(obj.segment)>=blk %else: no action needed: leave zeros
%         segment=obj.segment(blk);
% 
%         if ~isempty(segment.dat) %else: no action needed: leave zeros
%             %retrieve data
%             segdata=segmentData(obj.storemode,segment,obj.dataclass);
%             %range in full vector represented by these data
%             segrange=[1+(blk-1)*Bsize,(blk-1)*Bsize+length(segdata)]; 
%             
%             % -3- assign elements of expanded blocks to output vector
%             for k=1:size(rng,2)
%                 %find overlapping elements
%                 firstcommon = max(segrange(1),rng(1,k));
%                 lastcommon  = min(segrange(2),rng(2,k));
%                 %firstcommon and lastcommon are indices in full vector
%                 if firstcommon<=lastcommon
%                     %get index in segdata
%                     segIndx=(firstcommon-segrange(1)+1):(lastcommon-segrange(1)+1) ;
%                     %get external index
%                     extIndx=((firstcommon-rng(1,k)):(lastcommon-rng(1,k)))+extStart(k);
%                     %assign data
%                     data(extIndx)=segdata(segIndx);
%                 end           
%             end  
%             %close waitbar if it was created
%         end
%     end
%     close(hwait);
% end
