function [linindex,dims]=sub2indVector(sz,varargin)
% SUB2INDVECTOR Linear index from multiple vectorized subscripts.
% 
% CALL
%     [linindex,dims]=sub2indVector(SIZ,I1,I2,...,In)
%     
% INPUT
%     SIZ: size of matrix to index in
%     I1,I2,...,In: subscript array to be used
% 
% OUTPUT
%     linindex: [UINT32] linear index corresponding to I1,I2,...,In
%               ATTENTION! UINT32 variable will be rounded to integers
%               after division! ==> floor( uint8(6)/ 3) = 2!!
%               CONCLUSION: DO NOT DIVIDE INDICES!
%     dims: size of required dimensions
%           length(linindex)=prod(dims);
%           After:
%               [linindex,dims]=sub2indVector(size(M),I1,I2,...,In)
%           The following applies:        
%               M(I1,I2,...) 
%           is equivalent to: 
%               reshape(M(linindex),dims)
%         
%     SUB2INDVECTOR is used to determine the equivalent single index
%     corresponding to a given set of subscript values.
% 
%     This function issimilar to the Matlab sub2ind function but computes
%     the lineair indices to all combinations of the input indices I, J,
%     etc
%  
%     IND = SUB2INDVECTOR(SIZ,I,J) returns the linear index equivalent to the
%     row and column subscripts in the arrays I and J for an matrix of
%     size SIZ. 
%  
%     IND = SUB2INDVECTOR(SIZ,I1,I2,...,In) returns the linear index
%     equivalent to the N subscripts in the arrays I1,I2,...,In for an
%     array of size SIZ.
%  
% EXAMPLE: 
%     Consider the assignment
%     B=A([2 3],[4 5],[6 7])
%     B(:) now contains 8 elements of A
%     Their indices are 
%         2 4 6
%         3 4 6
%         2 5 6
%         3 5 6
%         2 4 7
%         3 4 7
%         2 5 7
%         3 5 7
%     Their lineair index in A are
%         2 (4-1)*size(A,1) (6-1)*size(A,1)*size(A,2)
%         3 (4-1)*size(A,1) (6-1)*size(A,1)*size(A,2)
%         etc.
% 
%     This function returns these linear indices
%
%See also:
%     ind2sub

% Nanne van der Zijpp
% Modelit

% if nargin==0
%     %testmode
%     linindex=sub2indVector([4 4],[1 3],[2 4])
%     return;
% end
    
%If number of subscripts is larger then number of dimensions append "1" for
%missing dimensions
sz(length(sz)+1:length(varargin))=1;

%compute the total number of required indices
for k=1:length(varargin) 
    dims(k)=length(varargin{k});
end
Ntot=prod(dims);
if Ntot==0
    %return empty index
    linindex=zeros(0,0,'uint32');
    return
end

%initialize the indices
linindex=zeros(Ntot,1,'uint32');

%initialize the patternlength
patternLen=1;

for k=1:length(varargin)    
    %build the pattern for this dimension
    %and repeat it a sufficient number of times
    %first dimension runns fast, second dimension slower, etc
    if k==1
        pattern=uint32(varargin{k}(:));
    else        
        pattern=(varargin{k}(:)-1)*prod(sz(1:k-1));        
        pattern = repmat(uint32(pattern'),patternLen,1);
        pattern = pattern(:);
    end
    N=Ntot/length(pattern);
    %triple memory required here 
    linindex=linindex+repmat(pattern,N,1);
    %     Memory friendly way: takes 2 times as much time
    %     if N==1
    %         linindex=linindex+pattern;
    %     else
    %         for nn=1:N
    %             rng=((nn-1)*length(pattern)+1):(nn*length(pattern));
    %             linindex(rng)=linindex(rng)+pattern;
    %         end
    %     end
    %%
    patternLen=patternLen*dims(k); %define the next pattern length
end
if length(dims)<2
    dims(2)=1; %size vectors must have at least 2 elements
end